<?php

/**
 * @file
 * Administrative forms for Autolink Tracking module.
 */

/**
 * Callback for link view form.
 */
function autolink_tracking_link_overview_page($link) {
  if (!empty($_POST['relationships'])) {
    $output = drupal_get_form('autolink_tracking_multiple_disable_link_confirm');
  }
  else {
    drupal_set_title(t('Relationships overview for <i>%keyword</i>: <i>%path</i>.', array('%keyword' => $link->keyword, '%path' => autolink_ui_get_path($link->link_path))));
    $output = drupal_get_form('autolink_tracking_link_overview', $link->lid);
  }
  return $output;
}

/**
 * Link overview form.
 */
function autolink_tracking_link_overview(&$form_state, $lid) {
  $header = array();
  $header[] = array('data' => t('Node'), 'field' => 'r.nid', 'sort' => 'asc');
  $header[] = array('data' => t('Title'), 'field' => 'n.title', 'sort' => 'asc');
  $header[] = t('Path');
  $header[] = array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc');

  $sql = "SELECT a.lid, a.keyword, r.rel_id, r.nid, r.relevance, n.title FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid LEFT JOIN {node} n ON r.nid = n.nid WHERE a.lid = $lid AND r.relevance <> ". AUTOLINK_RELATIONSHIP_DISABLED;
  $query_count = "SELECT COUNT(a.lid) FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE a.lid = $lid AND r.relevance <> ". AUTOLINK_RELATIONSHIP_DISABLED;

  $sql .= tablesort_sql($header);
  $result = pager_query($sql, 25, 0, $query_count);

  while ($link = db_fetch_object($result)) {
    $paths = array();
    if (isset($link->nid)) {
      $path = autolink_tracking_build_path($link->nid);

      $relationships[$link->rel_id] = '';
      $form['node'][$link->rel_id] = array('#value' => check_plain($link->nid));
      $form['title'][$link->rel_id] = array('#value' => l($link->title, "node/$link->nid"));
      $form['path'][$link->rel_id] = array('#value' => l($path, 'node/'. $link->nid));
      $form['relevance'][$link->rel_id] = array('#value' => check_plain($link->relevance));
    }
  }

  $form['relationships'] = array('#type' => 'checkboxes', '#options' => $relationships);
  $form['pager'] = array('#value' => theme('pager', NULL, 25, 0));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Remove relationships'));

  return $form;
}

/**
 * Form validation for outgoing links update form.
 */
function autolink_tracking_link_overview_validate($form, &$form_state) {
  $form_state['values']['relationships'] = array_filter($form_state['values']['relationships']);
  if (!count($form_state['values']['relationships'])) {
    form_set_error('', t('No relationships were selected.'));
  }
}

/**
 * Theme the incoming links overview as a sortable list of links.
 *
 * @ingroup themeable
 * @see autolink_overview_incoming_links()
 */
function theme_autolink_tracking_link_overview($form) {
  $header = array(
    theme('table_select_header_cell'),
    array('data' => t('Node'), 'field' => 'r.nid', 'sort' => 'asc'),
    array('data' => t('Title'), 'field' => 'n.title', 'sort' => 'asc'),
    t('Path'),
    array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc'),
  );

  if (isset($form['node']) && is_array($form['node'])) {
    foreach (element_children($form['node']) as $key) {
      $rows[] = array(
        drupal_render($form['relationships'][$key]),
        drupal_render($form['node'][$key]),
        drupal_render($form['title'][$key]),
        drupal_render($form['path'][$key]),
        drupal_render($form['relevance'][$key]),
      );
    }
  }
  else {
    $rows[] = array(array('data' => t('No relationships available.'), 'colspan' => '5'));
  }

  $output = theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

/**
 * Confirmation form for removing relationships from the incoming links form.
 */
function autolink_tracking_multiple_disable_link_confirm(&$form_state) {
  $edit = $form_state['post'];
  $form['relationships'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);

  foreach (array_filter($edit['relationships']) as $rel_id => $value) {
    $link = db_fetch_object(db_query('SELECT a.lid, a.group_type, r.rel_id, r.nid FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE r.rel_id = %d', $rel_id));
    $path = autolink_tracking_build_path($link->nid);
    $form['relationships'][$rel_id] = array(
      '#type' => 'hidden',
      '#value' => $rel_id,
      '#prefix' => '<li>',
      '#suffix' => check_plain($path) ."</li>\n",
    );
  }

  return confirm_form($form,
                      t('Are you sure you want to remove these relationships?'),
                      "admin/content/autolink/$link->group_type/link/$link->lid", t('This action cannot be undone.'),
                      t('Remove all'), t('Cancel'));
}

/**
 * Submit handler for removing links from the incoming links form.
 */
function autolink_tracking_multiple_disable_link_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    foreach ($form_state['values']['relationships'] as $rel_id => $value) {
      $link = db_fetch_object(db_query('SELECT a.lid, a.group_type, r.rel_id, r.nid FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE r.rel_id = %d', $rel_id));
      autolink_tracking_remove_relationship($rel_id);
      $node = node_load($link->nid);
      if (autolink_node_process($node, variable_get('autolink_total_link_limit', 'all'))) {
        node_save($node);
      }
    }
    drupal_set_message(t('The relationships have been removed.'));
  }

  $form_state['redirect'] = "admin/content/autolink/$link->group_type/link/$link->lid";

  return;
}
