<?php

/**
 * @file
 * Front end pages for Autolink Tracking module.
 */

/**
 * Callback function to build the related links page.
 */
function autolink_tracking_incoming_links_page($node) {
  if (!empty($_POST['relationships'])) {
    $output = drupal_get_form('autolink_tracking_multiple_disable_incoming_confirm');
  }
  else {
    drupal_set_title(t('Incoming links for <i>%title</i>', array('%title' => $node->title)));
    $output = drupal_get_form('autolink_tracking_incoming_filter_form');
    $output .= drupal_get_form('autolink_tracking_overview_incoming_links', $node->nid);
  }
  return $output;
}

/**
 * Form builder for incoming links filter form.
 *
 * @ingroup forms
 * @see autolink_tracking_incoming_filter_form_submit()
 */
function autolink_tracking_incoming_filter_form() {
  $session = &$_SESSION['autolink_incoming_filter'];
  $session = is_array($session) ? $session : array();
  $filters = autolink_tracking_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only links where'),
    '#theme' => 'autolink_tracking_filters',
  );
  foreach ($session as $filter) {
    list($type, $value) = $filter;
    // Merge an array of arrays into one if necessary.
    $options = $filters[$type]['options'];
    $params = array('%property' => $filters[$type]['title'] , '%value' => $options[$value]);
    if ($i++ > 0) {
      $form['filters']['current'][] = array('#value' => t('<em>and</em> where <strong>%property</strong> is <strong>%value</strong>', $params));
    }
    else {
      $form['filters']['current'][] = array('#value' => t('<strong>%property</strong> is <strong>%value</strong>', $params));
    }
  }

  foreach ($filters as $key => $filter) {
    $names[$key] = $filter['title'];
    $form['filters']['status'][$key] = array(
      '#type' => 'select',
      '#options' => $filter['options'],
    );
  }

  $form['filters']['filter'] = array(
    '#type' => 'radios',
    '#options' => $names,
  );
  $form['filters']['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => (count($session) ? t('Refine') : t('Filter')),
  );

  if (count($session)) {
    $form['filters']['buttons']['undo'] = array(
      '#type' => 'submit',
      '#value' => t('Undo'),
    );
    $form['filters']['buttons']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset'),
    );
  }

  drupal_add_js('misc/form.js', 'core');

  return $form;
}

/**
 * Process result from autolink administration filter form.
 */
function autolink_tracking_incoming_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];
  $filters = autolink_tracking_filters();

  switch ($op) {
    case t('Filter'): case t('Refine'):
      if (isset($form_state['values']['filter'])) {
        $filter = $form_state['values']['filter'];
        $options = $filters[$filter]['options'];
        if (isset($options[$form_state['values'][$filter]])) {
          $_SESSION['autolink_incoming_filter'][] = array($filter, $form_state['values'][$filter]);
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION['autolink_incoming_filter']);
      break;
    case t('Reset'):
      $_SESSION['autolink_incoming_filter'] = array();
      break;
    case t('Update'):
      return;
  }

  $form_state['redirect'] = 'node/'. arg(1) .'/autolink/incoming';
  return;
}

/**
 * Form builder to list incoming links.
 *
 * @ingroup forms
 * @see autolink_overview_incoming_links_submit()
 * @see theme_autolink_overview_incoming_links()
 */
function autolink_tracking_overview_incoming_links(&$form_state, $nid) {
  $groups = autolink_ui_get_groups();
  $filter = autolink_tracking_build_filter_query('incoming');

  $header = array();
  $header[] = array('data' => t('Link type'), 'field' => 'a.type');
  $header[] = array('data' => t('Group'), 'field' => 'a.group_type');
  $header[] = array('data' => t('Keyword'), 'field' => 'a.keyword');
  $header[] = t('Origin');
  $header[] = array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc');

  if (user_access('administer autolink')) {
    $header[] = t('Operations');
  }

  $sql = "SELECT a.lid, a.type, a.group_type, a.keyword, a.link_path, r.rel_id, r.nid, r.relevance FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE a.link_path = 'node/". $nid ."' AND r.relevance <> ". AUTOLINK_RELATIONSHIP_DISABLED . $filter['where'];
  $query_count = "SELECT COUNT(a.lid) FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE a.link_path = 'node/". $nid ."' AND r.relevance <> ". AUTOLINK_RELATIONSHIP_DISABLED . $filter['where'];

  $sql .= tablesort_sql($header);
  $result = pager_query($sql, 25, 0, $query_count);

  while ($link = db_fetch_object($result)) {
    $paths = array();
    if (isset($link->nid)) {
      $path = autolink_tracking_build_path($link->nid);

      $relationships[$link->rel_id] = '';
      $form['type'][$link->rel_id] = array('#value' => check_plain($link->type));
      $form['group_type'][$link->rel_id] = array('#value' => l($groups[$link->group_type]->name, "admin/content/autolink/$link->group_type"));
      $form['keyword'][$link->rel_id] = array('#value' => l($link->keyword, "admin/content/autolink/$link->group_type/link/$link->lid"));
      $form['origination'][$link->rel_id] = array('#value' => l($path, 'node/'. $link->nid));
      $form['relevance'][$link->rel_id] = array('#value' => check_plain($link->relevance));
      if (user_access('administer autolink')) {
        $form['edit'][$link->rel_id] = array('#value' => l(t('edit link'), "admin/content/autolink/$link->group_type/edit/link/$link->lid"));
      }
    }
  }

  $form['relationships'] = array('#type' => 'checkboxes', '#options' => $relationships);
  $form['pager'] = array('#value' => theme('pager', NULL, 25, 0));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Remove relationships'));

  return $form;
}

/**
 * Form validation for outgoing links update form.
 */
function autolink_tracking_overview_incoming_links_validate($form, &$form_state) {
  $form_state['values']['relationships'] = array_filter($form_state['values']['relationships']);
  if (!count($form_state['values']['relationships'])) {
    form_set_error('', t('No links were selected.'));
  }
}

/**
 * Theme the incoming links overview as a sortable list of links.
 *
 * @ingroup themeable
 * @see autolink_overview_incoming_links()
 */
function theme_autolink_tracking_overview_incoming_links($form) {
  $header = array(
    theme('table_select_header_cell'),
    array('data' => t('Type'), 'field' => 'a.type'),
    array('data' => t('Group'), 'field' => 'a.group_type'),
    array('data' => t('Keyword'), 'field' => 'a.keyword'),
    t('Origin'),
    array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc'),
  );

  if (user_access('administer autolink')) {
    $header[] = array('data' => t('Operations'), 'colspan' => '1');
  }

  if (isset($form['keyword']) && is_array($form['keyword'])) {
    foreach (element_children($form['keyword']) as $key) {
      $row = array();
      $row[] = drupal_render($form['relationships'][$key]);
      $row[] = drupal_render($form['type'][$key]);
      $row[] = drupal_render($form['group_type'][$key]);
      $row[] = drupal_render($form['keyword'][$key]);
      $row[] = drupal_render($form['origination'][$key]);
      $row[] = drupal_render($form['relevance'][$key]);
      if (user_access('administer autolink')) {
        $row[] = drupal_render($form['edit'][$key]);
      }

      $rows[] = $row;
    }
  }
  else {
    $rows[] = array(array('data' => t('No relationships available.'), 'colspan' => '6'));
  }

  $output = theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

/**
 * Confirmation form for removing relationships from the incoming links form.
 */
function autolink_tracking_multiple_disable_incoming_confirm(&$form_state) {
  $edit = $form_state['post'];
  $nid = arg(1);
  $form['relationships'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);

  foreach (array_filter($edit['relationships']) as $rel_id => $value) {
    $relationship = db_fetch_object(db_query('SELECT r.rel_id, r.nid FROM {autolink_relationship} r WHERE r.rel_id = %d', $rel_id));
    $path = autolink_tracking_build_path($relationship->nid);
    $form['relationships'][$rel_id] = array(
      '#type' => 'hidden',
      '#value' => $rel_id,
      '#prefix' => '<li>',
      '#suffix' => check_plain($path) ."</li>\n",
    );
  }

  return confirm_form($form,
                      t('Are you sure you want to remove these relationships?'),
                      "node/$nid/autolink/incoming", t('This action cannot be undone.'),
                      t('Remove all'), t('Cancel'));
}

/**
 * Submit handler for removing links from the incoming links form.
 */
function autolink_tracking_multiple_disable_incoming_confirm_submit($form, &$form_state) {
  $nid = arg(1);
  if ($form_state['values']['confirm']) {
    foreach ($form_state['values']['relationships'] as $rel_id => $value) {
      $link = db_fetch_object(db_query('SELECT a.lid, r.rel_id, r.nid FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE r.rel_id = %d', $rel_id));
      autolink_tracking_remove_relationship($rel_id);

      // Remove disabled links from the originating node.
      $node = node_load($link->nid);
      if (autolink_node_process($node, variable_get('autolink_total_link_limit', 'all'))) {
        node_save($node);
      }
    }
    drupal_set_message(t('The relationships have been removed.'));
  }

  $form_state['redirect'] = "node/$nid/autolink/incoming";

  return;
}

/**
 * Callback function to build the outgoing related links page.
 */
function autolink_tracking_outgoing_links_page($node) {
  if (!empty($_POST['relationships'])) {
    $output = drupal_get_form('autolink_tracking_multiple_disable_outgoing_confirm');
  }
  else {
    drupal_set_title(t('Outgoing links for <i>%title</i>', array('%title' => $node->title)));
    $output = drupal_get_form('autolink_tracking_outgoing_filter_form');
    $output .= drupal_get_form('autolink_tracking_overview_outgoing_links', $node->nid);
  }
  return $output;
}

/**
 * Form builder for outgoing links filter form.
 *
 * @ingroup forms
 * @see autolink_tracking_outgoing_filter_form_submit()
 */
function autolink_tracking_outgoing_filter_form() {
  $session = &$_SESSION['autolink_outgoing_filter'];
  $session = is_array($session) ? $session : array();
  $filters = autolink_tracking_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only links where'),
    '#theme' => 'autolink_tracking_filters',
  );
  foreach ($session as $filter) {
    list($type, $value) = $filter;
    // Merge an array of arrays into one if necessary.
    $options = $filters[$type]['options'];
    $params = array('%property' => $filters[$type]['title'] , '%value' => $options[$value]);
    if ($i++ > 0) {
      $form['filters']['current'][] = array('#value' => t('<em>and</em> where <strong>%property</strong> is <strong>%value</strong>', $params));
    }
    else {
      $form['filters']['current'][] = array('#value' => t('<strong>%property</strong> is <strong>%value</strong>', $params));
    }
  }

  foreach ($filters as $key => $filter) {
    $names[$key] = $filter['title'];
    $form['filters']['status'][$key] = array(
      '#type' => 'select',
      '#options' => $filter['options'],
    );
  }

  $form['filters']['filter'] = array(
    '#type' => 'radios',
    '#options' => $names,
  );
  $form['filters']['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => (count($session) ? t('Refine') : t('Filter')),
  );

  if (count($session)) {
    $form['filters']['buttons']['undo'] = array(
      '#type' => 'submit',
      '#value' => t('Undo'),
    );
    $form['filters']['buttons']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset'),
    );
  }

  drupal_add_js('misc/form.js', 'core');

  return $form;
}

/**
 * Process result from autolink administration filter form.
 */
function autolink_tracking_outgoing_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];
  $filters = autolink_tracking_filters();

  switch ($op) {
    case t('Filter'): case t('Refine'):
      if (isset($form_state['values']['filter'])) {
        $filter = $form_state['values']['filter'];
        $options = $filters[$filter]['options'];
        if (isset($options[$form_state['values'][$filter]])) {
          $_SESSION['autolink_outgoing_filter'][] = array($filter, $form_state['values'][$filter]);
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION['autolink_outgoing_filter']);
      break;
    case t('Reset'):
      $_SESSION['autolink_outgoing_filter'] = array();
      break;
    case t('Update'):
      return;
  }

  $form_state['redirect'] = 'node/'. arg(1) .'/autolink/outgoing';
  return;
}

/**
 * Form builder to list outgoing links from a specific node.
 *
 * @ingroup forms
 * @see autolink_tracking_overview_outgoing_links_submit()
 * @see theme_autolink_tracking_overview_outgoing_links()
 */
function autolink_tracking_overview_outgoing_links(&$form_state, $nid) {
  $groups = autolink_ui_get_groups();
  $filter = autolink_tracking_build_filter_query('outgoing');

  $header = array();
  $header[] = array('data' => t('Type'), 'field' => 'a.type');
  $header[] = array('data' => t('Link group'), 'field' => 'a.group_type');
  $header[] = array('data' => t('Keyword'), 'field' => 'a.keyword');
  $header[] = t('Destination');
  $header[] = array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc');

  if (user_access('administer autolink')) {
    $header[] = t('Operations');
  }

  $sql = 'SELECT a.lid, a.type, a.group_type, a.keyword, a.link_path, r.rel_id, r.relevance FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE r.nid = '. $nid .' AND r.relevance <> '. AUTOLINK_RELATIONSHIP_DISABLED . $filter['where'];
  $query_count = 'SELECT COUNT(a.lid) FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE r.nid = '. $nid .' AND r.relevance <> '. AUTOLINK_RELATIONSHIP_DISABLED . $filter['where'];

  $sql .= tablesort_sql($header);
  $result = pager_query($sql, 25, 0, $query_count, $filter['args']);

  while ($link = db_fetch_object($result)) {
    $destination = autolink_ui_get_path($link->link_path);

    $relationships[$link->rel_id] = '';
    $form['type'][$link->rel_id] = array('#value' => check_plain($link->type));
    $form['group_type'][$link->rel_id] = array('#value' => l($groups[$link->group_type]->name, "admin/content/autolink/$link->group_type"));
    $form['keyword'][$link->rel_id] = array('#value' => l($link->keyword, "admin/content/autolink/$link->group_type/link/$link->lid"));
    $form['destination'][$link->rel_id] = array('#value' => l($destination, $link->link_path));
    $form['relevance'][$link->rel_id] = array('#value' => check_plain($link->relevance));
    if (user_access('administer autolink')) {
      $form['edit'][$link->rel_id] = array('#value' => l(t('edit link'), "admin/content/autolink/$link->group_type/edit/link/$link->lid"));
    }
  }

  $form['relationships'] = array('#type' => 'checkboxes', '#options' => $relationships);
  $form['pager'] = array('#value' => theme('pager', NULL, 25, 0));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Remove relationships'));

  return $form;
}

/**
 * Form validation for outgoing links update form.
 */
function autolink_tracking_overview_outgoing_links_validate($form, &$form_state) {
  $form_state['values']['relationships'] = array_filter($form_state['values']['relationships']);
  if (!count($form_state['values']['relationships'])) {
    form_set_error('', t('No links were selected.'));
  }
}

/**
 * Theme the outgoing linkgs overview as a sortable list of links.
 *
 * @ingroup themeable
 * @see autolink_overview_outgoing_links()
 */
function theme_autolink_tracking_overview_outgoing_links($form) {
  $header = array(
    theme('table_select_header_cell'),
    array('data' => t('Type'), 'field' => 'a.type'),
    array('data' => t('Link group'), 'field' => 'a.group_type'),
    array('data' => t('Keyword'), 'field' => 'a.keyword'),
    t('Destination'),
    array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc'),
  );

  if (user_access('administer autolink')) {
    $header[] = array('data' => t('Operations'), 'colspan' => '1');
  }

  if (isset($form['keyword']) && is_array($form['keyword'])) {
    foreach (element_children($form['keyword']) as $key) {
      $row = array();
      $row[] = drupal_render($form['relationships'][$key]);
      $row[] = drupal_render($form['type'][$key]);
      $row[] = drupal_render($form['group_type'][$key]);
      $row[] = drupal_render($form['keyword'][$key]);
      $row[] = drupal_render($form['destination'][$key]);
      $row[] = drupal_render($form['relevance'][$key]);
      if (user_access('administer autolink')) {
        $row[] = drupal_render($form['edit'][$key]);
      }

      $rows[] = $row;
    }
  }
  else {
    $rows[] = array(array('data' => t('No relationships available.'), 'colspan' => '7'));
  }

  $output = theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

/**
 * Confirmation form for removing relationships from the outgoing links form.
 *
 * @ingroup forms
 * @see autolink_tracking_overview_outgoing_links_submit()
 */
function autolink_tracking_multiple_disable_outgoing_confirm(&$form_state) {
  $edit = $form_state['post'];

  $form['relationships'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);
  foreach (array_filter($edit['relationships']) as $rel_id => $value) {
    $relationship = db_fetch_object(db_query('SELECT a.keyword, a.argument, r.nid FROM {autolink_relationship} r LEFT JOIN {autolink_link} a ON r.lid = a.lid WHERE r.rel_id = %d', $rel_id));
    $form['relationships'][$rel_id] = array(
      '#type' => 'hidden',
      '#value' => $rel_id,
      '#prefix' => '<li>',
      '#suffix' => check_plain($relationship->argument) ."</li>\n",
    );
  }

  return confirm_form($form,
                      t('Are you sure you want to remove these relationships?'),
                      "node/$nid/autolink/outgoing", t('This action cannot be undone.'),
                      t('Remove all'), t('Cancel'));
}

/**
 * Submit handler for removing relationships from the outgoing links form.
 *
 * @ingroup forms
 * @see autolink_tracking_multiple_disable_outgoing_confirm()
 */
function autolink_tracking_multiple_disable_outgoing_confirm_submit($form, &$form_state) {
  $nid = arg(1);
  if ($form_state['values']['confirm']) {
    foreach ($form_state['values']['relationships'] as $rel_id => $value) {
      autolink_tracking_remove_relationship($rel_id);
    }

    // Process the node to remove the disabled links.
    $node = node_load($nid);
    if (autolink_node_process($node, variable_get('autolink_total_link_limit', 'all'))) {
      node_save($node);
    }
    drupal_set_message(t('The relationships have been removed.'));
  }

  $form_state['redirect'] = "node/$nid/autolink/outgoing";

  return;
}

/**
 * Theme outgoing links filter form.
 *
 * @ingroup themeable
 */
function theme_autolink_tracking_filter_form($form) {
  $output = '<div id="autolink-tracking-filter">';
  $output .= drupal_render($form['filters']);
  $output .= '</div>';
  $output .= drupal_render($form);
  return $output;
}

/**
 * Theme autolink administration filter selector.
 *
 * @ingroup themeable
 */
function theme_autolink_tracking_filters($form) {
  $output = '<ul class="clear-block">';

  if (!empty($form['current'])) {
    foreach (element_children($form['current']) as $key) {
      $output .= '<li>'. drupal_render($form['current'][$key]) .'</li>';
    }
  }

  $output .= '<li><dl class="multiselect">'. (!empty($form['current']) ? '<dt><em>'. t('and') .'</em> '. t('where') .'</dt>' : '') .'<dd class="a">';

  foreach (element_children($form['filter']) as $key) {
    $output .= drupal_render($form['filter'][$key]);
  }

  $output .= '</dd>';
  $output .= '<dt>'. t('is') .'</dt><dd class="b">';

  foreach (element_children($form['status']) as $key) {
    $output .= drupal_render($form['status'][$key]);
  }

  $output .= '</dd>';
  $output .= '</dl>';

  $output .= '<div class="container-inline" id="autolink-tracking-buttons">'. drupal_render($form['buttons']) .'</div>';
  $output .= '</li></ul>';

  return $output;
}

/**
 * Defines filters for outgoing links filter form.
 */
function autolink_tracking_filters() {
  $filters = array();

  // Link types filter.
  $link_types = autolink_ui_link_types();
  if (count($link_types)) {
    $filters['link_type'] = array(
      'title' => t('link type'),
      'where' => "a.type = '%s'",
      'options' => $link_types,
      'join' => '',
    );
  }

  // Groups filter.
  $result = db_query('SELECT * FROM {autolink_group} ORDER BY name');
  while ($group = db_fetch_object($result)) {
    $groups[$group->type] = $group->name;
  }

  if (count($groups)) {
    $filters['link_group'] = array(
      'title' => t('link group'),
      'where' => "a.group_type = '%s'",
      'options' => $groups,
      'join' => '',
    );
  }

  return $filters;
}

/**
 * Build query for autolink admin filters based on session.
 */
function autolink_tracking_build_filter_query($dir) {
  $filters = autolink_tracking_filters();

  // Build query.
  $where = $args = $join = array();
  foreach ($_SESSION['autolink_'. $dir .'_filter'] as $filter) {
    list($key, $value) = $filter;
    $where[] = $filters[$key]['where'];
    $args[] = $value;
    $join[] = $filters[$key]['join'];
  }

  $where = !empty($where) ? ' AND '. implode(' AND ', $where) : '';
  $join = !empty($join) ? ' '. implode(' ', array_unique($join)) : '';

  return array('where' => $where, 'join' => $join, 'args' => $args);
}
