<?php

/**
 * @file
 * Administrative page callbacks for the autolink module.
 */

/**
 * Settings form for autolink.
 *
 * @ingroup forms
 * @see autolink_ui_form_settings_submit()
 */
function autolink_ui_form_settings(&$form_state, $edit = array()) {
  $form['general_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('General settings'),
    '#collapsible' => TRUE,
    '#weight' => -10,
  );

  // General settings.
  $form['general_settings']['autolink_link_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Enable link types'),
    '#options' => autolink_ui_link_types(),
    '#default_value' => array_keys(autolink_ui_setting('autolink_link_types', autolink_ui_link_types(), 1)),
    '#description' => t('Note: disabling a link type will remove all link definitions '.
                        'from that link type.'),
  );

  // Add pattern processing with Token for link titles.
  if (module_exists('token')) {
    $form['general_settings']['autolink_tokens'] = array(
      '#type' => 'item',
      '#value' => t('Title patterns are used to generate link text when generating '.
                    'links in blocks or as part of the <i>title</i> attribute in links '.
                    'that are generated in node content.'));

    if (variable_get('autolink_link_types_node', 1) == AUTOLINK_SETTING_ENABLED) {
      $form['general_settings']['autolink_node_title'] = array(
        '#type' => 'textfield',
        '#title' => t('Pattern replacement for node links'),
        '#default_value' => variable_get('autolink_node_title', '[title]'),
        '#description' => t('Title patterns are used to generate link text when generating '.
                            'links in blocks or as part of the <i>title</i> attribute in links '.
                            'that are generated in node content.'),
      );
      $form['general_settings']['node_token_help'] = array(
        '#type' => 'fieldset',
        '#title' => t('Replacement patterns'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );
      $form['general_settings']['node_token_help']['node_help'] = array(
        '#value' => theme('token_help', 'node'),
      );
    }

    if (variable_get('autolink_link_types_user', 1) == AUTOLINK_SETTING_ENABLED) {
      $form['general_settings']['autolink_user_title'] = array(
        '#type' => 'textfield',
        '#title' => t('Pattern replacement for user links'),
        '#default_value' => variable_get('autolink_user_title', '[user]'),
      );
      $form['general_settings']['user_token_help'] = array(
        '#type' => 'fieldset',
        '#title' => t('Replacement patterns'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );
      $form['general_settings']['user_token_help']['user_help'] = array(
        '#value' => theme('token_help', 'user'),
      );
    }

    if (variable_get('autolink_link_types_taxonomy', 1) == AUTOLINK_SETTING_ENABLED) {
      $form['general_settings']['autolink_taxonomy_title'] = array(
        '#type' => 'textfield',
        '#title' => t('Pattern replacement for taxonomy term links'),
        '#default_value' => variable_get('autolink_taxonomy_title', '[term]'),
      );
      $form['general_settings']['taxonomy_token_help'] = array(
        '#type' => 'fieldset',
        '#title' => t('Replacement patterns'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );
      $form['general_settings']['taxonomy_token_help']['taxonomy_help'] = array(
        '#value' => theme('token_help', 'taxonomy'),
      );
    }
  }

  if (variable_get('autolink_link_types_node', 1) == AUTOLINK_SETTING_ENABLED) {
    $form['general_settings']['autolink_link_tagging'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enable link tagging on node edit forms'),
      '#default_value' => variable_get('autolink_link_tagging', 1),
      '#description' => t('With link tagging enabled, link definitions that link to a given node '.
                          'can be managed from the node edit form. This setting only applies to '.
                          '<i>node</i> links.'),
    );
  }

  $form['general_settings']['autolink_display_alias'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display aliases on administrative forms'),
    '#default_value' => variable_get('autolink_display_alias', 1),
    '#description' => t('Organize administrative forms by displaying aliases whenever '.
                        'possible. Note: This setting has no affect on the paths used when '.
                        'building links. Autolink will always use normal Drupal paths in links.'),
  );

  $defaults = autolink_ui_get_settings();

  $form['global'] = array(
    '#type' => 'fieldset',
    '#title' => t('Global link generation'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => -9,
    '#description' => t('Global settings allow site administrators more control over '.
                        'the location and frequency of link generation in node content.'),
  );
  $form['global']['autolink_link_limit'] = array(
    '#type' => 'select',
    '#title' => t('Limit links per keyword per node'),
    '#options' => array(
      'all' => 'all',
      '1' => '1',
      '2' => '2',
      '3' => '3',
      '4' => '4',
      '5' => '5',
      '6' => '6',
      '7' => '7',
      '8' => '8',
      '9' => '9',
      '10' => '10',
    ),
    '#default_value' => variable_get('autolink_link_limit', 'all'),
    '#description' => t('Providing a maximum number of links allowed will ensure that only that number '.
                        'of links for a given keyword will be generated per node.'),
  );
  $form['global']['autolink_total_link_limit'] = array(
    '#type' => 'select',
    '#title' => t('Limit total links per node'),
    '#options' => array(
      'all' => 'all',
      '1' => '1',
      '2' => '2',
      '3' => '3',
      '4' => '4',
      '5' => '5',
      '6' => '6',
      '7' => '7',
      '8' => '8',
      '9' => '9',
      '10' => '10',
      '11' => '11',
      '12' => '12',
      '13' => '13',
      '14' => '14',
      '15' => '15',
      '16' => '16',
      '17' => '17',
      '18' => '18',
      '19' => '19',
      '20' => '20',
    ),
    '#default_value' => variable_get('autolink_total_link_limit', 'all'),
    '#descrption' => t('Limit the total number of links that Autolink will generate on any individual node.'),
  );
  $form['global']['autolink_teaser_links'] = array(
    '#type' => 'checkbox',
    '#title' => t('Generate links in teasers'),
    '#default_value' => variable_get('autolink_teaser_links', 1),
  );
  $form['global']['autolink_remove_links'] = array(
    '#type' => 'checkbox',
    '#title' => t('Remove links during node edit'),
    '#default_value' => variable_get('autolink_remove_links', 1),
  );

  // Automatic link generation settings.
  $form['automatic'] = array(
    '#type' => 'fieldset',
    '#title' => t('Automatic link generation'),
    '#weight' => -8,
    '#collapsible' => TRUE,
    '#collapsed' => variable_get('autolink_auto_generate', 0) == 1 ? FALSE : TRUE,
  );
  $form['automatic']['autolink_auto_generate'] = array(
    '#type' => 'checkbox',
    '#title' => t('Automatically generate links from node titles, users, or terms'),
    '#default_value' => variable_get('autolink_auto_generate', 0),
    '#description' => t('Automatically generate links when nodes, users, or terms '.
                        'are located in node content.<br />' .
                        'Note: Link type and node type settings are available when '.
                        'this setting is enabled.'),
  );

  if (variable_get('autolink_auto_generate', 0) == AUTOLINK_SETTING_ENABLED) {
    if (module_exists('taxonomy')) {
      $type_options = array('node' => 'Node', 'user' => 'User', 'taxonomy' => 'Taxonomy Term');
    }
    else {
      $type_options = array('node' => 'Node', 'user' => 'User');
    }

    $form['automatic']['autolink_auto_link_types'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Limit automatic generation by link type'),
      '#options' => $type_options,
      '#default_value' => array_keys(autolink_ui_setting('autolink_auto_link_types', autolink_ui_link_types(), 0)),
    );

    if (variable_get('autolink_auto_link_types_node', 0) == AUTOLINK_SETTING_ENABLED) {
      $form['automatic']['autolink_auto_node_types'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Limit automatic link generation by content type'),
        '#options' => node_get_types('names'),
        '#default_value' => array_keys(autolink_ui_setting('autolink_auto_node_types', node_get_types('names'), 0)),
        '#description' => t('Select which content types automatic links may be applied to.'),
      );
    }

    if (variable_get('autolink_auto_link_type_user', 0) == AUTOLINK_SETTING_ENABLED) {
      $form['automatic']['autolink_auto_user_roles'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Limit automatic link destinations by user role'),
        '#options' => user_roles(),
        '#default_value' => array_keys(autolink_ui_setting('autolink_auto_user_roles', user_roles(), 0)),
        '#description' => t('User roles that are not selected will not be considered '.
                            'for automatic link generation. i.e. If <i>administrator</i> is not '.
                            'selected then Autolink will not seek out user name under the '.
                            'role of <i>administrator</i>'),
      );
    }
    if (variable_get('autolink_auto_link_type_taxonomy', 0) == AUTOLINK_SETTING_ENABLED) {
      $form['automatic']['autolink_auto_vocabularies'] = array(
        '#type' => 'select',
        '#title' => t('Limit autolink link detection by vocabulary'),
        '#multiple' => TRUE,
        '#options' => autolink_ui_get_vocabularies(),
        '#default_value' => array_keys(autolink_ui_setting('autolink_auto_vocabularies', autolink_ui_get_vocabularies(), 0)),
        '#description' => t('Terms within the selected vocabularies will be sought in '.
                            'node content.'),
      );
    }
  }

  // Default link settings.
  $form['default'] = array(
    '#type' => 'fieldset',
    '#title' => t('Link attributes'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => -7,
    '#description' => t('Default settings will be used to populate new link definition forms and '.
                        'when generating links in node content.'),
  );
  $form['default']['autolink_case_sensitivity'] = array(
    '#type' => 'checkbox',
    '#title' => t('Case-sensitivity'),
    '#default_value' => $defaults->case_sensitivity,
    '#description' => t('Keywords will only be replaced with links if they match the case of the keyword. '.
                        'This setting serves only as a default, as the case-sensitivity attribute can '.
                        'be managed on a link-by-link basis on the link add/edit form.'),
  );
  $form['default']['autolink_style'] = array(
    '#type' => 'textfield',
    '#title' => t('Style'),
    '#default_value' => $defaults->style,
    '#description' => t('Although it is recommended that you use a stylesheet and class, inline styles may '.
                        'be used for additional link attributes.'),
  );
  $form['default']['autolink_class'] = array(
    '#type' => 'textfield',
    '#title' => t('Class'),
    '#default_value' => $defaults->class,
    '#description' => t('New link definitions can automatically be assigned classes. To assign more than '.
                        'one class enter a space separated list.'),
  );
  $form['default']['autolink_internal_target'] = array(
    '#type' => 'select',
    '#title' => t('Target (internal links)'),
    '#options' => array(
      'none' => 'none',
      '_blank' => '_blank',
      '_parent' => '_parent',
      '_self' => '_self',
      '_top' => '_top',
    ),
    '#default_value' => $defaults->internal_target,
    '#description' => t('Specifies where to open internally linked documents.'),
  );
  $form['default']['autolink_external_target'] = array(
    '#type' => 'select',
    '#title' => t('Target (external links)'),
    '#options' => array(
      'none' => 'none',
      '_blank' => '_blank',
      '_parent' => '_parent',
      '_self' => '_self',
      '_top' => '_top',
    ),
    '#default_value' => $defaults->external_target,
    '#description' => t('Specifies where to open externally linked documents.'),
  );
  $form['default']['autolink_external_relationship'] = array(
    '#type' => 'checkbox',
    '#title' => t('Set <i>rel</i> attribute to <i>nofollow</i> for external links'),
    '#default_value' => variable_get('autolink_external_relationship', 1),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['buttons']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset to defaults'),
    '#weight' => 10,
  );

  return $form;
}

/**
 * Validation handler for the Autolink settings form.
 */
function autolink_ui_form_settings_validate($form, &$form_state) {
  foreach ($form_state['values']['autolink_link_types'] as $type => $set) {
    if ($set) {
      if ($type == 'search' && !module_exists('search')) {
        form_set_error('autolink_link_types', t('You must enable the search module before enabling the <i>search</i> link type.'));
      }
      elseif ($type == 'taxonomy' && !module_exists('taxonomy')) {
        form_set_error('autolink_link_types', t('You must enable the taxonomy module before enabling the <i>taxonomy</i> link type.'));
      }
    }
  }
}

/**
 * Submit handler for Autolink settings form.
 *
 * This handler sets miltiple value field values by creating individual variables
 * for each value. This makes node and link type settings much easier to work with
 * on other forms and in processing functions and is cleaner than using the
 * 'array_filter' field.
 */
function autolink_ui_form_settings_submit($form, &$form_state) {
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : '';

  // Exclude unnecessary elements.
  unset($form_state['values']['submit'], $form_state['values']['reset'], $form_state['values']['form_id'], $form_state['values']['op'], $form_state['values']['form_token'], $form_state['values']['form_build_id']);

  foreach ($form_state['values'] as $key => $value) {
    // The form was reset to defaults. Delete all variables.
    if ($op == t('Reset to defaults')) {
      if ($form_state['values'][$key]['#type'] == 'checkboxes') {
        foreach ($form_state['values'][$key] as $type => $values) {
          variable_del($key .'_'. $type);
        }
      }
      elseif ($key == 'autolink_auto_vocabularies') {
        foreach (autolink_ui_get_vocabularies() as $vid => $name) {
          variable_del('autolink_auto_vocabularies_'. $vid);
        }
      }
      else {
        variable_del($key);
      }
    }
    else {
      // Set checkboxes variables.
      if ($key == 'autolink_link_types' || $key == 'autolink_auto_link_types' || $key == 'autolink_auto_node_types' || $key == 'autolink_auto_user_roles') {
        foreach ($form_state['values'][$key] as $type => $state) {
          $variable = $key .'_'. $type;
          if ($state) {
            variable_set($variable, 1);
          }
          // Remove link definitions from a link group if that link group was disabled.
          elseif ($key == 'autolink_link_types' && variable_get($variable, 1) == AUTOLINK_SETTING_ENABLED) {
            $result = db_query("SELECT lid FROM {autolink_link} WHERE type = '%s'", $type);
            while ($lid = db_result($result)) {
              autolink_ui_link_delete($lid);
            }
            variable_set($variable, 0);
          }
          else {
            variable_set($variable, 0);
          }
        }
      }
      // Set taxonomy variables.
      elseif ($key == 'autolink_auto_vocabularies') {
        foreach (autolink_ui_get_vocabularies() as $vid => $name) {
          variable_set('autolink_auto_vocabularies_'. $vid, 0);
        }
        foreach ($form_state['values']['autolink_auto_vocabularies'] as $type => $state) {
          variable_set('autolink_auto_vocabularies_'. $vid, 1);
        }
      }
      else {
        variable_set($key, $value);
      }
    }
  }

  if ($op == t('Reset to defaults')) {
    drupal_set_message(t('The configuration options have been reset to their default values.'));
  }
  else {
    drupal_set_message(t('The configuration options have been saved.'));
  }

  cache_clear_all();
  drupal_rebuild_theme_registry();
}

/**
 * Callback for displaying the groups overview form.
 */
function autolink_ui_admin_groups() {
  module_load_include('inc', 'autolink_ui', 'includes/autolink_ui.filters');
  module_load_include('inc', 'autolink_ui', 'includes/autolink_ui.operations');
  if (!empty($_POST['groups']) && isset($_POST['operation']) && ($_POST['operation'] == 'delete')) {
    $output = drupal_get_form('autolink_ui_group_multiple_delete_confirm');
  }
  else {
    $output = drupal_get_form('autolink_ui_filter_form');
    $output .= drupal_get_form('autolink_ui_overview_groups');
  }
  return $output;
}

/**
 * Form builder for Autolink groups overview filters.
 *
 * @ingroup forms
 * @see autolink_ui_filter_form_submit()
 */
function autolink_ui_filter_form() {
  $session = &$_SESSION['autolink_overview_filter'];
  $session = is_array($session) ? $session : array();
  $filters = autolink_ui_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only link groups where'),
    '#theme' => 'autolink_ui_filters',
  );
  foreach ($session as $filter) {
    list($type, $value) = $filter;
    // Merge an array of arrays into one if necessary.
    $options = $filters[$type]['options'];
    $params = array('%property' => $filters[$type]['title'] , '%value' => $options[$value]);
    if ($i++ > 0) {
      $form['filters']['current'][] = array('#value' => t('<em>and</em> where <strong>%property</strong> is <strong>%value</strong>', $params));
    }
    else {
      $form['filters']['current'][] = array('#value' => t('<strong>%property</strong> is <strong>%value</strong>', $params));
    }
  }

  foreach ($filters as $key => $filter) {
    $names[$key] = $filter['title'];
    $form['filters']['status'][$key] = array(
      '#type' => 'select',
      '#options' => $filter['options'],
    );
  }

  $form['filters']['filter'] = array(
    '#type' => 'radios',
    '#options' => $names,
  );
  $form['filters']['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => (count($session) ? t('Refine') : t('Filter')),
  );

  if (count($session)) {
    $form['filters']['buttons']['undo'] = array(
      '#type' => 'submit',
      '#value' => t('Undo'),
    );
    $form['filters']['buttons']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset'),
    );
  }

  drupal_add_js('misc/form.js', 'core');

  return $form;
}

/**
 * Theme Autolink administration filter form.
 *
 * @ingroup themeable
 */
function theme_autolink_ui_filter_form($form) {
  $output = '<div id="autolink-admin-filter">';
  $output .= drupal_render($form['filters']);
  $output .= '</div>';
  $output .= drupal_render($form);
  return $output;
}

/**
 * Theme autolink administration filter selector.
 *
 * @ingroup themeable
 */
function theme_autolink_ui_filters($form) {
  $output = '<ul class="clear-block">';

  if (!empty($form['current'])) {
    foreach (element_children($form['current']) as $key) {
      $output .= '<li>'. drupal_render($form['current'][$key]) .'</li>';
    }
  }

  $output .= '<li><dl class="multiselect">'. (!empty($form['current']) ? '<dt><em>'. t('and') .'</em> '. t('where') .'</dt>' : '') .'<dd class="a">';

  foreach (element_children($form['filter']) as $key) {
    $output .= drupal_render($form['filter'][$key]);
  }

  $output .= '</dd>';
  $output .= '<dt>'. t('is') .'</dt><dd class="b">';

  foreach (element_children($form['status']) as $key) {
    $output .= drupal_render($form['status'][$key]);
  }

  $output .= '</dd>';
  $output .= '</dl>';

  $output .= '<div class="container-inline" id="autolink-admin-buttons">'. drupal_render($form['buttons']) .'</div>';
  $output .= '</li></ul>';

  return $output;
}


/**
 * Process result from autolink administration filter form.
 */
function autolink_ui_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];
  $filters = autolink_ui_filters();

  switch ($op) {
    case t('Filter'): case t('Refine'):
      if (isset($form_state['values']['filter'])) {
        $filter = $form_state['values']['filter'];
        $options = $filters[$filter]['options'];
        if (isset($options[$form_state['values'][$filter]])) {
          $_SESSION['autolink_overview_filter'][] = array($filter, $form_state['values'][$filter]);
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION['autolink_overview_filter']);
      break;
    case t('Reset'):
      $_SESSION['autolink_overview_filter'] = array();
      break;
    case t('Update'):
      return;
  }

  $form_state['redirect'] = 'admin/content/autolink/';
  return;
}

/**
 * Form builder; Autolink administration page.
 *
 * @ingroup forms
 * @see autolink_ui_overview_groups_validate()
 * @see autolink_ui_overview_groups_submit()
 */
function autolink_ui_overview_groups() {
  $filter = autolink_ui_build_filter_query();

  $header = array();
  $header[] = array('data' => t('Group'), 'field' => 'g.name', 'sort' => 'asc');
  $header[] = array('data' => t('Description'), 'field' => 'g.description');
  $header[] = t('Node Types');
  $header[] = array('data' => t('Status'), 'field' => 'g.status');
  $header[] = t('Operations');

  if ($filter['join'] != "") {
    $sql = 'SELECT DISTINCT g.type, g.name, g.description, g.status FROM {autolink_group} g LEFT JOIN {autolink_group_node_type} t ON g.type = t.group_type '. $filter['join'] .' '. $filter['where'];
    $query_count = 'SELECT COUNT(DISTINCT g.type) FROM {autolink_group} g LEFT JOIN {autolink_group_node_type} t ON g.type = t.group_type '. $filter['join'] .' '. $filter['where'];
  }
  else {
    $sql = 'SELECT g.type, g.name, g.description, g.status FROM {autolink_group} g '. $filter['where'];
    $query_count = 'SELECT COUNT(g.type) FROM {autolink_group} g '. $filter['where'];
  }

  $sql .= tablesort_sql($header);

  $result = pager_query($sql, 25, 0, $query_count, $filter['args']);

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update link groups'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );

  // Add our own operations first.
  $options = array();
  $ops = autolink_ui_group_operations();
  foreach ($ops as $operation => $array) {
    $options[$operation] = $array['label'];
  }

  // Invoke hook_autolink_operations() to get a list of external operations.
  foreach (module_invoke_all('autolink_group_operations') as $operation => $array) {
    $options[$operation] = $array['label'];
  }

  $form['options']['operation'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => 'activate',
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  $destination = drupal_get_destination();

  $status = array(t('disabled'), t('active'));
  $groups = array();

  while ($group = db_fetch_object($result)) {
    $groups[$group->type] = '';
    $form['group'][$group->type] = array('#value' => l($group->name, "admin/content/autolink/$group->type"));
    $form['description'][$group->type] = array('#value' => $group->description);

    $node_types = array();
    $types_result = db_query("SELECT n.type, n.name FROM {node_type} n INNER JOIN {autolink_group_node_type} t ON n.type = t.node_type WHERE t.group_type = '%s'", $group->type);
    while ($type = db_fetch_object($types_result)) {
      $node_types[$type->type] = $type->name;
    }
    asort($node_types);
    $form['types'][$group->type][0] = array('#value' => implode(', ', $node_types));
    $form['status'][$group->type] =  array('#value' => $status[$group->status]);
    $form['operations'][$group->type]['edit'] = array('#value' => l(t('edit group'), "admin/content/autolink/edit/group/$group->type", array('query' => $destination)));
    $form['operations'][$group->type]['add'] = array('#value' => l(t('add link'), "admin/content/autolink/$group->type/add/link", array('query' => "admin/content/autolink/$group->type")));
  }

  $form['groups'] = array(
    '#type' => 'checkboxes',
    '#options' => $groups
  );
  $form['pager'] = array('#value' => theme('pager', NULL, 25, 0));

  return $form;
}

/**
 * Form validation for Autolink groups administration update form.
 */
function autolink_ui_overview_groups_validate($form, &$form_state) {
  $form_state['values']['groups'] = array_filter($form_state['values']['groups']);
  if (!count($form_state['values']['groups'])) {
    form_set_error('', t('No groups were selected.'));
  }
}

/**
 * Submit handler for Autolink groups administration update form.
 */
function autolink_ui_overview_groups_submit($form, &$form_state) {
  $operations = autolink_ui_group_operations($form_state);
  $operations += module_invoke_all('autolink_group_operations', $form_state);
  $operation = $operations[$form_state['values']['operation']];
  // Filter out unchecked groups.
  $groups = array_filter($form_state['values']['groups']);
  if ($function = $operation['callback']) {
    // Add in callback arguments if present.
    if (isset($operation['callback arguments'])) {
      $args = array_merge(array($groups), $operation['callback arguments']);
    }
    else {
      $args = array($groups);
    }
    // Call the function based on the operation that was selected.
    call_user_func_array($function, $args);

    drupal_set_message(t('The update has been performed.'));
  }
}

/**
 * Theme Autolink administration groups overview.
 *
 * @ingroup themeable
 */
function theme_autolink_ui_overview_groups($form) {
  // Overview table:
  $header = array(
    theme('table_select_header_cell'),
    array('data' => t('Group'), 'field' => 'g.type', 'sort' => 'asc'),
    array('data' => t('Description'), 'field' => 'g.description'),
    t('Content types'),
    array('data' => t('Status'), 'field' => 'g.status'),
    t('Operations'),
  );

  $output = drupal_render($form['options']);
  if (isset($form['group']) && is_array($form['group'])) {
    foreach (element_children($form['group']) as $key) {
      $rows[] = array(
        drupal_render($form['groups'][$key]),
        drupal_render($form['group'][$key]),
        drupal_render($form['description'][$key]),
        drupal_render($form['types'][$key]),
        drupal_render($form['status'][$key]),
        drupal_render($form['operations'][$key]),
      );
    }
  }
  else {
    $rows[] = array(array('data' => t('No link groups available.'), 'colspan' => '6'));
  }

  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

/**
 * Display form for adding and editing link groups.
 *
 * @ingroup forms
 * @see autolink_ui_form_group_submit()
 */
function autolink_ui_form_group(&$form_state, $edit = array()) {
  $form['identification'] = array(
    '#type' => 'fieldset',
    '#title' => t('Identification'),
    '#collapsible' => TRUE,
  );
  $form['identification']['status'] = array(
    '#type' => 'select',
    '#title' => t('Status'),
    '#default_value' => isset($edit['status']) ? $edit['status'] : 1,
    '#options' => array(0 => t('Disabled'), 1 => t('Active')),
  );
  $form['identification']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Group name'),
    '#default_value' => isset($edit['name']) ? $edit['name'] : '',
    '#maxlength' => 255,
    '#description' => t('The human-readable name for this link group.'),
    '#required' => TRUE,
  );
  $form['identification']['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => isset($edit['description']) ? $edit['description'] : '',
    '#description' => t('Description of the link group.'),
  );
  $form['identification']['in_content'] = array(
    '#type' => 'checkbox',
    '#title' => t('Generate links from this group in node content'),
    '#default_value' => isset($edit['in_content']) ? $edit['in_content'] : '',
    '#description' => t('Autolink will search for keywords from this link group in '.
                        'node content and replace them with proper links.'),
  );
  $form['nodes'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content types'),
    '#collapsible' => TRUE,
  );
  $form['nodes']['node_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Content types'),
    '#default_value' => isset($edit['types']) ? $edit['types'] : array(),
    '#options' => array_map('check_plain', node_get_types('names')),
    '#description' => t('Select content types for this link group to monitor.'),
  );

  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));
  if (isset($edit['type'])) {
    $form['delete'] = array('#type' => 'submit', '#value' => t('Delete'));
    $form['type'] = array('#type' => 'value', '#value' => $edit['type']);
  }
  else {
    // Add a validate function to ensure group names are not duplicated.
    $form['#valdate'][] = 'autolink_ui_form_group_duplicate_validate';
  }

  return $form;
}

/**
 * Validate function for add link group form.
 */
function autolink_ui_form_group_duplicate_validate($form, &$form_state) {
  $group = $form_state['values']['name'];
  $result = db_query("SELECT * FROM {autolink_group} WHERE name = '%s'", $group);
  if (db_result($result)) {
    form_set_error('name', t('A link group with the name %name already exists.', array('%name' => $group)));
  }
}

/**
 * Submit function for add link group form.
 */
function autolink_ui_form_group_submit($form, &$form_state) {
  // Fix up the nodes array to remove unchecked nodes.
  $form_state['values']['types'] = array_filter($form_state['values']['node_types']);
  switch (autolink_ui_group_save($form_state['values'])) {
    case SAVED_NEW:
      drupal_set_message(t('Created new link group %name.', array('%name' => $form_state['values']['name'])));
      watchdog('autolink', 'Created new link group %name.', array('%name' => $form_state['values']['name']), WATCHDOG_NOTICE, l(t('edit'), 'admin/content/autolink/edit/group/'. $form_state['values']['type']));
      break;
    case SAVED_UPDATED:
      drupal_set_message(t('Updated link group %name.', array('%name' => $form_state['values']['name'])));
      watchdog('autolink', 'Updated link group %name.', array('%name' => $form_state['values']['name']), WATCHDOG_NOTICE, l(t('edit'), 'admin/content/autolink/edit/group/'. $form_state['values']['type']));
      break;
  }

  $form_state['type'] = $form_state['values']['type'];
  $form_state['redirect'] = 'admin/content/autolink';
  return;
}

/**
 * Page to edit a link group.
 */
function autolink_ui_admin_group_edit($group) {
  if ((isset($_POST['op']) && $_POST['op'] == t('Delete')) || isset($_POST['confirm'])) {
    return drupal_get_form('autolink_ui_group_confirm_delete', $group->type);
  }
  return drupal_get_form('autolink_ui_form_group', (array)$group);
}

/**
 * Form builder for the link group delete confirmation form.
 *
 * @ingroup forms
 * @see autolink_ui_group_confirm_delete_submit()
 */
function autolink_ui_group_confirm_delete(&$form_state, $type) {
  $group = autolink_ui_group_load($type);

  $form['type'] = array('#type' => 'value', '#value' => $type);
  $form['name'] = array('#type' => 'value', '#value' => $group->name);
  return confirm_form($form,
                  t('Are you sure you want to delete the link group %name?',
                  array('%name' => $group->name)),
                  'admin/content/autolink',
                  t('Deleting a link group will delete all the link definitions within it. This action cannot be undone.'),
                  t('Delete'),
                  t('Cancel'));
}

/**
 * Submit handler to delete a link group after confirmation.
 *
 * @see autolink_ui_group_confirm_delete()
 */
function autolink_ui_group_confirm_delete_submit($form, &$form_state) {
  $status = autolink_ui_group_delete($form_state['values']['type']);
  drupal_set_message(t('Deleted link group %name.', array('%name' => $form_state['values']['name'])));
  watchdog('autolink', 'Deleted link group %name.', array('%name' => $form_state['values']['name']), WATCHDOG_NOTICE);
  $form_state['redirect'] = 'admin/content/autolink';
  return;
}

/**
 * Callback for generating the links overview form.
 */
function autolink_ui_admin_links($list, $group) {
  module_load_include('inc', 'autolink_ui', 'includes/autolink_ui.filters');
  module_load_include('inc', 'autolink_ui', 'includes/autolink_ui.operations');
  if (!empty($_POST['links']) && isset($_POST['operation']) && ($_POST['operation'] == 'delete')) {
    $output = drupal_get_form('autolink_ui_link_multiple_delete_confirm');
  }
  else {
    drupal_set_title(t('Link definitions for %group', array('%group' => $group->name)));
    $output = drupal_get_form('autolink_ui_overview_links', (array) $group);
  }
  return $output;
}

/**
 * Form builder; Autolink administration page.
 *
 * @ingroup forms
 * @see autolink_ui_overview_links_validate()
 * @see autolink_ui_overview_links_submit()
 */
function autolink_ui_overview_links($form_state, $group) {
  $header = array();
  $header[] = array('data' => t('Type'), 'field' => 'a.type');
  $header[] = array('data' => t('Keyword'), 'field' => 'a.keyword', 'sort' => 'asc');
  $header[] = t('Destination');
  $header[] = t('Operations');

  $sql = "SELECT a.lid, a.group_type, a.type, a.keyword, a.link_path, a.query FROM {autolink_link} a WHERE a.group_type = '". $group['type'] ."' AND a.lid <> 0";
  $query_count = "SELECT COUNT(a.lid) FROM {autolink_link} a WHERE a.group_type = '". $group['type'] ."' AND a.lid <> 0";

  $sql .= tablesort_sql($header);

  $result = pager_query($sql, 25, 0, $query_count, $filter['args']);

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $options = array();

  // Add our own operations first.
  $ops = autolink_ui_link_operations();
  foreach ($ops as $operation => $array) {
    $options[$operation] = $array['label'];
  }

  // Invoke hook_autolink_operations() to get a list of external operations.
  foreach (module_invoke_all('autolink_link_operations') as $operation => $array) {
    $options[$operation] = $array['label'];
  }

  $form['options']['operation'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => 'activate',
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  $destination = drupal_get_destination();

  $links = array();
  $link_types = autolink_ui_link_types();
  while ($link = db_fetch_object($result)) {
    $destination = autolink_ui_get_path($link->link_path);
    if ($link->type == 'node_search' || $link->type == 'user_search') {
      $destination = $link->link_path .'/'. $link->query;
    }

    // Link the keyword to its relationship page if tracking is enabled.
    $keyword = module_exists('autolink_tracking') ? l($link->keyword, "admin/content/autolink/$link->group_type/link/$link->lid") : $link->keyword;

    $links[$link->lid] = '';
    $form['type'][$link->lid] = array('#value' => $link_types[$link->type]);
    $form['keyword'][$link->lid] = array('#value' => $keyword);
    $form['destination'][$link->lid] =  array('#value' => l($destination, $link->link_path));
    $form['operations'][$link->lid] = array('#value' => l(t('edit'), "admin/content/autolink/$link->group_type/edit/link/$link->lid", array('query' => $destination)));
  }

  $form['links'] = array(
    '#type' => 'checkboxes',
    '#options' => $links
  );
  $form['pager'] = array('#value' => theme('pager', NULL, 25, 0));
  $form['group'] = array('#type' => 'value', '#value' => $group);

  return $form;
}

/**
 * Form validation for Autolink administration update form.
 */
function autolink_ui_overview_links_validate($form, &$form_state) {
  $form_state['values']['links'] = array_filter($form_state['values']['links']);
  if (!count($form_state['values']['links'])) {
    form_set_error('', t('No link definitions were selected.'));
  }
}

/**
 * Submit handler for Autolink administration update form.
 */
function autolink_ui_overview_links_submit($form, &$form_state) {
  $operations = autolink_ui_link_operations($form_state);
  $operations += module_invoke_all('autolink_link_operations', $form_state);
  $operation = $operations[$form_state['values']['operation']];
  // Filter out unchecked links.
  $links = array_filter($form_state['values']['links']);
  if ($function = $operation['callback']) {
    // Add in callback arguments if present.
    if (isset($operation['callback arguments'])) {
      $args = array_merge(array($links), $operation['callback arguments']);
    }
    else {
      $args = array($links);
    }
    // Call the function based on the operation that was selected.
    call_user_func_array($function, $args);

    drupal_set_message(t('The update has been performed.'));
  }
}

/**
 * Theme Autolink administration overview.
 *
 * @ingroup themeable
 */
function theme_autolink_ui_overview_links($form) {
  // Overview table:
  $header = array(
    theme('table_select_header_cell'),
    array('data' => t('Type'), 'field' => 'a.type'),
    array('data' => t('Keyword'), 'field' => 'a.keyword', 'sort' => 'asc'),
    t('Destination'),
    t('Operations'),
  );

  $output = drupal_render($form['options']);
  if (isset($form['keyword']) && is_array($form['keyword'])) {
    foreach (element_children($form['keyword']) as $key) {
      $rows[] = array(
        drupal_render($form['links'][$key]),
        drupal_render($form['type'][$key]),
        drupal_render($form['keyword'][$key]),
        drupal_render($form['destination'][$key]),
        drupal_render($form['operations'][$key]),
      );
    }
  }
  else {
    $rows[] = array(array('data' => t('No link definitions available.'), 'colspan' => '5'));
  }

  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

/**
 * Form for adding or editing a link definition.
 *
 * This function uses the AHAH Helper module to build a dynamic link definition
 * form. When the AHAH submit function is called and the form is rebuilt this
 * function builds link type specific form elements based on data retrieved with
 * the link type that has been selected.
 */
function autolink_ui_form_link($form_state, $group, $edit = array()) {
  _autolink_ui_include();

  $form = array();
  $form['#group'] = (array)$group;

  ahah_helper_register($form, $form_state);

  // Prepare link types for the form.
  $select_type = array('select' => t('--Select a link type--'));
  $link_types = autolink_ui_setting('autolink_link_types', autolink_ui_link_types(), 1);
  asort($link_types);

  if ($link_types) {
    $link_type_options = array_merge($select_type, $link_types);
  }
  else {
    $link_type_options = $select_type;
    drupal_set_message(t('There are currently no link types enabled. To enable a ' .
                         'link type please visit the Autolink settings page.'), 'warning');
  }

  $keyword_disabled = FALSE;
  $keyword_required = TRUE;

  // If this is an edit form then load the link data and store the link object
  // in the form so other modules can access it using hook_form_alter().
  if (isset($edit['lid'])) {
    $link = autolink_ui_link_load($edit['lid']);
    $form['link'] = array('#type' => 'value', '#value' => $link);
    $keyword_disabled = TRUE;
    $keyword_required = FALSE;
  }

  if (!isset($form_state['storage']['link_wrapper']['type'])) {
    $link_type = isset($edit['type']) ? $edit['type'] : 'select';
  }
  else {
    $link_type = $form_state['storage']['link_wrapper']['type'];
  }

  $form['link_wrapper'] = array(
    '#prefix' => '<div id="link-wrapper">',
    '#suffix' => '</div>',
    '#tree'   => TRUE,
  );
  // If there is only one link type then set that as the default instead of 'select'.
  $form['link_wrapper']['type'] = array(
    '#type' => 'select',
    '#title' => t('Link to'),
    '#options' => $link_type_options,
    '#default_value' => $link_type,
    '#ahah' => array(
      'event' => 'change',
      'path' => ahah_helper_path(array('link_wrapper')),
      'wrapper' => 'link-wrapper',
      'effect' => 'fade',
    ),
  );
  $form['link_wrapper']['type_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Set link type'),
    '#submit' => array('ahah_helper_generic_submit'),
    '#attributes' => array('class' => 'no-js'),
  );

  // Only if a link type has been selected do we show the link definition form.
  if ($link_type != 'select') {
    $form['link_wrapper']['definition'] = array(
      '#type' => 'fieldset',
      '#title' => t('Definition'),
    );
    $form['link_wrapper']['definition']['keyword'] = array(
      '#type' => 'textfield',
      '#title' => t('Keyword(s)'),
      '#default_value' => isset($edit['keyword']) ? $edit['keyword'] : '',
      '#required' => $keyword_required,
      '#disabled' => $keyword_disabled,
      '#description' => $keyword_disabled == TRUE ? NULL :
        t('The keyword you would like converted into a link. This can be more than one word'),
    );
    $form['link_wrapper']['definition']['case_sensitive'] = array(
      '#type' => 'checkbox',
      '#title' => t('Case-sensitive keyword matching'),
      '#default_value' => isset($edit['case_sensitive']) ? $edit['case_sensitive'] : variable_get('autolink_case_sensitivity', 0),
      '#description' => t('Words will only be matched if they match letter case. i.e. <i>AutoLink</i> ' .
                          'is not the same as <i>autolink</i> if case-sensitivity is enabled.'),
    );

    // We don't want the keyword to be edited once it is stored in the database
    // since we search for those keywords in nodes on update if the path is changed.
    // Since Drupal will not save the default value of 'keyword' while it is
    // disabled we have to create a hidden field to store our value.
    if ($keyword_disabled == TRUE && isset($edit['keyword'])) {
      $form['link_wrapper']['stored_keyword'] = array(
        '#type' => 'value',
        '#value' => $edit['keyword'],
      );
    }

    // Initialize method form elements to prevent errors.
    $form['link_wrapper']['definition']['method_wrapper'] = array();
    $form['link_wrapper']['definition']['method_wrapper']['method'] = array();

    $methods = autolink_ui_form_methods();

    // This is a special handler for to add a dropdown for link types that have
    // more than one method of definition.
    if (count($methods[$link_type]) > 1) {

      // Set the method for edit forms or from the form storage.
      if (!isset($form_state['storage']['link_wrapper']['definition']['method_wrapper']['method'])) {
        $selected_method = isset($edit['method']) ? $edit['method'] : 'no_method';
      }
      else {
        $selected_method = $form_state['storage']['link_wrapper']['definition']['method_wrapper']['method'];
      }

      $method_options['no_method'] = t('--Select a method--');
      if (!empty($link_type)) {
        $method_options += $methods[$link_type];
      }

      $form['link_wrapper']['definition']['method_wrapper'] = array(
        '#prefix' => '<div id="method-wrapper">',
        '#suffix' => '</div>',
        '#tree'   => TRUE,
      );
      $form['link_wrapper']['definition']['method_wrapper']['method'] = array(
        '#type' => 'select',
        '#title' => t('Selection method'),
        '#options' => $method_options,
        '#default_value' => $selected_method,
        '#required' => TRUE,
        '#ahah' => array(
          'event' => 'change',
          'path' => ahah_helper_path(array('link_wrapper', 'definition', 'method_wrapper')),
          'wrapper' => 'method-wrapper',
          'effect' => 'fade',
        ),
      );

      // Check to ensure a method is selected and that the method belongs to the
      // currently selected link type before building the form. This prevents form
      // elements from other link types from appearing on the wrong link type forms.
      if ($selected_method != 'no_method' && isset($methods[$link_type][$selected_method])) {
        $edit['method'] = isset($edit['method']) ? $edit['method'] : $form_state['values']['link_wrapper']['definition']['method_wrapper']['method'];
  
        // Prepare callbacks and arguments to build form elements based on the selection method.
        $method = _autolink_ui_get_method($selected_method);
        $form['method'] = array('#type' => 'value', '#value' => $selected_method);
        $method['callback arguments'] = isset($method['callback arguments']) ? $method['callback arguments'] : array();
  
        // Call the selection method specific form builder callback.
        $type_form = _autolink_ui_method_form_preprocess($method['callback'], $edit, $method['callback arguments']);
  
        if ($type_form) {
          $form['link_wrapper']['definition']['method_wrapper'] += $type_form;
        }
      }
    }
    else {
      // There is no selected method, so get the callbacks in a foreach loop.
      $method = _autolink_ui_link_type_method($link_type);
      $form['method'] = array('#type' => 'value', '#value' => $selected_method);
      $method['callback arguments'] = isset($method['callback arguments']) ? $method['callback arguments'] : array();

      // Call the selection method specific form builder callback.
      $type_form = _autolink_ui_method_form_preprocess($method['callback'], $edit, $method['callback arguments']);
      if ($type_form) {
        $form['link_wrapper']['definition'] += $type_form;
      }
    }
  }

  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));
  $form['group_type'] = array('#type' => 'value', '#value' => $group->type);
  if (isset($edit['lid'])) {
    $form['delete'] = array('#type' => 'submit', '#value' => t('Delete'));
    $form['lid'] = array('#type' => 'value', '#value' => $edit['lid']);
  }

  return $form;
}

/**
 * Validation handler for the link edit form.
 *
 * Here we ensure the link does not contain inappropriate characters. Also, since
 * we are using disabled and hidden fields, we have to do extra form validation
 * here because we cannot use the 'required' => TRUE property on a disabled field.
 *
 * @see autolink_ui_form_link()
 */
function autolink_ui_form_link_validate($form, &$form_state) {
  _autolink_ui_include();
  $values = _autolink_ui_merge_link_form_values($form_state['values']);

  if (empty($values['keyword']) && !isset($values['stored_keyword'])) {
    form_set_error('keyword', t('You must enter a keyword.'));
  }
  elseif (!empty($values['keyword']) && !eregi("^[a-zA-Z0-9 -]{2,40}$", $values['keyword'])) {
    form_set_error('keyword', t('The keyword contains an unacceptable value.'));
  }

  // Call validate callbacks specified by the selection method.
  $method = _autolink_ui_get_method($form_values['method']);
  if (function_exists($method['validate'])) {
    $values['form_values'] = $form_values;
    if (isset($method['callback arguments'])) {
      $values += $method['callback arguments'];
    }
    call_user_func_array($method['validate'], $values);
  }
}

/**
 * Final submit function for the link form.
 */
function autolink_ui_form_link_submit($form, &$form_state) {
  $link = _autolink_ui_merge_link_form_values($form_state['values']);

  switch (autolink_ui_link_save($link)) {
    case SAVED_NEW:
      drupal_set_message(t('Created new <i>%link_type</i> type link for keyword <i>%keyword</i>.', array('%link_type' => $link['type'], '%keyword' => $link['keyword'])));
      watchdog('autolink', 'Created new <i>%link_type</i> type link for keyword <i>%keyword</i>.', array('%link_type' => $link['type'], '%keyword' => $link['keyword']), WATCHDOG_NOTICE, l(t('edit'), 'admin/content/autolink/'. $link['group_type'] .'/edit/link/'. $link['lid']));
      break;
    case SAVED_UPDATED:
      drupal_set_message(t('Updated <i>%link_type</i> type link for keyword <i>%keyword</i>.', array('%link_type' => $form_state['values']['link_wrapper']['type'], '%keyword' => $link['stored_keyword'])));
      watchdog('autolink', 'Updated link for keyword <i>%keyword</i>.', array('%keyword' => $link['stored_keyword']), WATCHDOG_NOTICE, l(t('edit'), 'admin/content/autolink/'. $link['group_type'] .'/edit/link/'. $link['lid']));
      break;
  }

  $form_state['lid'] = $form_state['values']['lid'];
  drupal_goto('admin/content/autolink/'. $link['group_type']);

  return;
}

/**
 * Page to add a link.
 */
function autolink_ui_add_link_page($group) {
  drupal_set_title(t('Add link definition to group %group', array('%group' => $group->name)));
  return drupal_get_form('autolink_ui_form_link', $group);
}

/**
 * Page to edit a link.
 *
 * @param $link
 *   A link object.
 *
 * @see autolink_ui_form_link()
 * @see autolink_ui_confirm_delete()
 */
function autolink_ui_admin_link_edit($group, $link) {
  if ((isset($_POST['op']) && $_POST['op'] == t('Delete')) || isset($_POST['confirm'])) {
    return drupal_get_form('autolink_ui_confirm_link_delete', $link->lid);
  }
  return drupal_get_form('autolink_ui_form_link', (array)$group, (array)$link);
}

/**
 * Form builder for the link delete confirmation form.
 *
 * @param $lid
 *   The ID of a link.
 *
 * @ingroup forms
 * @see autolink_ui_confirm_link_delete_submit()
 */
function autolink_ui_confirm_link_delete(&$form_state, $lid) {
  $link = autolink_ui_link_load($lid);

  $form['type'] = array(
    '#type' => 'value',
    '#value' => 'link',
  );
  $form['lid'] = array(
    '#type' => 'value',
    '#value' => $link->lid,
  );
  $form['keyword'] = array(
    '#type' => 'value',
    '#value' => $link->keyword,
  );
  return confirm_form($form,
                  t('Are you sure you want to delete the link %keyword?',
                  array('%keyword' => $link->keyword)),
                  "admin/content/autolink/$link->group_type",
                  t('This action cannot be undone.'),
                  t('Delete'),
                  t('Cancel'));
}

/**
 * Submit handler to delete a link after confirmation.
 *
 * @see autolink_ui_confirm_link_delete()
 */
function autolink_ui_confirm_link_delete_submit($form, &$form_state) {
  // Delete the link and record the deletion
  $status = autolink_ui_link_delete($form_state['values']['lid']);

  drupal_set_message(t('Deleted link definition for keyword <i>%link</i>.', array('%link' => $form_state['values']['keyword'])));
  watchdog('autolink', 'Deleted link definition for keyword <i>%link</i>.', array('%link' => $form_state['values']['keyword']), WATCHDOG_NOTICE);
  $form_state['redirect'] = 'admin/content/autolink';

  return;
}

/**
 * Invokes method specific form builder callbacks with arguments.
 *
 * @param $callback
 *   A string representing the current selection method.
 * @param $edit = array()
 *   Form values that need to be set as default values on form elements.
 * @param $callback_arguments = array()
 *   An optional array of arguments to pass to the form callback function.
 *
 * @return
 *   Form elements populated with defaults and ready for display.
 */
function _autolink_ui_method_form_preprocess($callback, $edit = array(), $callback_arguments = array()) { 
  $edit['argument'] = isset($edit['link_path']) ? html_entity_decode($edit['link_path']) : '';
  $arguments['edit'] = $edit;
  if (!empty($callback_arguments)) {
    $arguments += $callback_arguments;
  }
  foreach (call_user_func_array($callback, $arguments) as $field => $value) {
    $elements[$field] = $value;
  }
  return $elements;
}

/**
 * Merges AHAH form values into a readable $link array in preparation for database insert.
 *
 * @return
 *   Merged form values that can be processed for validation or inserted into the database.
 */
function _autolink_ui_merge_link_form_values($form_values) {
  // Because of AHAH we need to add the proper form elements to a link array of form data.
  $link = $form_values;
  $link += $form_values['link_wrapper'];
  $link += $form_values['link_wrapper']['definition'];

  if (isset($form_values['link_wrapper']['definition']['method_wrapper'])) {
    $link += $form_values['link_wrapper']['definition']['method_wrapper'];
  }

  return $link;
}
