<?php

/**
 * @file
 * Functions for the 'taxonomy' link type for use with the Autolink module.
 */

/**
 * Implements hook_autolink_LINK_TYPE_form_elements().
 */
function autolink_ui_autolink_taxonomy_form_elements($edit) {
  $form = array();

  if ($edit['method'] == 'dropdown') {
    if (isset($edit['link_path'])) {
      $args = explode('/', $edit['link_path']);
    }

    $terms = _autolink_ui_taxonomy_get_terms();

    $select['select'] = t('--Select a term--');
    $select += $terms;

    $form['argument'] = array(
      '#type' => 'select',
      '#title' => t('Select a term'),
      '#required' => TRUE,
      '#options' => $select,
      '#default_value' => isset($args[2]) ? $args[2] : 'select',
    );
  }
  elseif ($edit['method'] == 'terms') {
    if (isset($edit['link_path']) && !empty($edit['link_path'])) {

      // Get the term names from tids by parsing the path.
      $args = explode('/', $edit['link_path']);
      $tids = explode(',', $args[2]);

      foreach ($tids as $tid) {
        $term = taxonomy_get_term($tid);
        $terms[$tid] = $term->name;
      }

      asort($terms);
      $terms_default = implode(', ', $terms);
    }

    $form['argument'] = array(
      '#type' => 'textfield',
      '#title' => t('Taxonomy term(s)'),
      '#required' => TRUE,
      '#default_value' => isset($terms_default) ? $terms_default : '',
      '#description' => t('Enter a comma separated list of taxonomy terms to which the anchor should link.'),
    );
    $form['operator'] = array(
      '#type' => 'radios',
      '#title' => t('Operation'),
      '#options' => array(
        'and' => 'and',
        'or' => 'or',
      ),
      '#default_value' => isset($edit['operator']) ? $edit['operator'] : '',
    );
  }

  return $form;
}

/**
 * Callback function for building the taxonomy term selection form.
 */
function autolink_ui_taxonomy_select_method($edit) {
  $form = array();
  if (isset($edit['link_path'])) {
    $args = explode('/', $edit['link_path']);
  }

  $terms = _autolink_ui_taxonomy_get_terms();

  $select['select'] = t('--Select a term--');
  $select += $terms;

  $form['argument'] = array(
    '#type' => 'select',
    '#title' => t('Select a term'),
    '#required' => TRUE,
    '#options' => $select,
    '#default_value' => isset($args[2]) ? $args[2] : 'select',
  );
  return $form;
}

/**
 * Callback function for building the taxonomy term multiple form.
 */
function autolink_ui_taxonomy_multiple_method($edit) {
  $form = array();
  if (isset($edit['link_path']) && !empty($edit['link_path'])) {

    // Get the term names from tids by parsing the path.
    $args = explode('/', $edit['link_path']);
    $tids = explode(',', $args[2]);

    foreach ($tids as $tid) {
      $term = taxonomy_get_term($tid);
      $terms[$tid] = $term->name;
    }

    asort($terms);
    $terms_default = implode(', ', $terms);
  }

  $form['argument'] = array(
    '#type' => 'textfield',
    '#title' => t('Taxonomy term(s)'),
    '#required' => TRUE,
    '#default_value' => isset($terms_default) ? $terms_default : '',
    '#description' => t('Enter a comma separated list of taxonomy terms to which the anchor should link.'),
  );
  $form['operator'] = array(
    '#type' => 'radios',
    '#title' => t('Operation'),
    '#options' => array(
      'and' => 'and',
      'or' => 'or',
    ),
    '#default_value' => isset($edit['operator']) ? $edit['operator'] : '',
  );
  return $form;
}

/**
 * Implements hook_autolink_LINK_TYPE_validate().
 */
function autolink_ui_autolink_taxonomy_validate($form_values) {
  if ($form_values['method'] == 'dropdown' && $form_values['argument'] == 'select') {
    form_set_error('argument', t('You must select a term.'));
  }
  elseif ($form_values['method'] == 'terms') {
    $terms = drupal_explode_tags($form_values['argument']);

    // Ensure an operation is selected if multiple terms were entered.
    if (isset($terms[1]) && empty($form_values['operator'])) {
      form_set_error('operation', t('You must select an operation for multiple term definitions.'));
    }

    // Check to ensure terms exist.
    foreach ($terms as $term) {
      if (!autolink_ui_taxonomy_term_exists($term)) {
        form_set_error('argument', t('Cannot locate term data for \'%term\'. Please enter another term.', array('%term' => $term)));
      }
    }
    unset($term);
  }
}

/**
 * Validation callback for the taxonomy term select form.
 */
function autolink_ui_taxonomy_select_method_validate($form_values) {
  if ($form_values['argument'] == 'select') {
    form_set_error('argument', t('You must select a term.'));
  }
}

/**
 * Validation callback for the multiple term form.
 */
function autolink_ui_taxonomy_multiple_method_validate($form_values) {
  $terms = drupal_explode_tags($form_values['argument']);

  // Ensure an operation is selected if multiple terms were entered.
  if (isset($terms[1]) && empty($form_values['operator'])) {
    form_set_error('operator', t('You must select an operation for multiple term definitions.'));
  }

  // Check to ensure terms exist.
  foreach ($terms as $term) {
    if (!autolink_ui_taxonomy_term_exists($term)) {
      form_set_error('argument', t('Cannot locate term data for \'%term\'. Please enter another term.', array('%term' => $term)));
    }
  }
  unset($term);
}

/**
 * Implements hook_autolink_LINK_TYPE_alter().
 *
 * Converts the link path to be saved to a taxonomy page. Form data is used to
 * create a proper link path linking to the results page.
 *
 * @param $link
 *   An associative array of link form data.
 * @param $method
 *   The selection method used to create the link.
 */
function autolink_ui_autolink_taxonomy_alter(&$link, $method) {
  if ($link['method'] == 'term_select') {
    $link['link_path'] = 'taxonomy/term/'. $link['argument'];
  }
  elseif ($link['method'] == 'term_multiple') {
    $terms = drupal_explode_tags($link['argument']);

    // Get term IDs for use in urls.
    foreach ($terms as $name) {
      $result = db_query("SELECT * FROM {term_data} WHERE name = '%s'", $name);
      while ($term = db_fetch_object($result)) {
        $tids[] = $term->tid;
      }
    }

    // If this is only one term then we don't need an operation.
    if (!isset($terms[1])) {
      $arg = $tids[0];
    }
    else {
      // The operation field is a special field created for filtering AND and OR statements.
      switch ($link['operator']) {
        case 'and':
          $arg = implode(',', $tids);
          break;
        case 'or':
          $arg = implode('+', $tids);
          break;
      }
    }
    $link['link_path'] = 'taxonomy/term/'. $arg;
  }
}

/**
 * Returns an array of all taxonomy terms.
 */
function _autolink_ui_taxonomy_get_terms() {
  $terms = array();
  $result = db_query('SELECT * FROM {term_data}');
  while ($term = db_fetch_object($result)) {
    $terms[$term->tid] = $term->name;
  }
  asort($terms);
  return $terms;
}

/*
 * Helper function to determine whether a term exists.
 *
 * This function is used on taxonomy link form validation to ensure that the user is
 * entering terms that exist. The terms are taken from a comma separated list of keywords
 * and validated by querying the {term_date} table.
 *
 * @param $term
 *   A term to check for.
 *
 * @return
 *   TRUE if the term exists, FALSE otherwise.
 */
function autolink_ui_taxonomy_term_exists($term) {
  $result = db_query("SELECT * FROM {term_data} WHERE name = '%s'", $term);
  return (db_fetch_object($result)) ? TRUE : FALSE;
}
